/* nvram.c */

#include "product.h"
#include "board.h"
#include "synthcalcs.h"

#include <stdlib.h>
#include <assert.h>


/* Pin manipulation functions */
#include <avr/io.h>
#include <util/delay.h>
//#include <avr/pgmspace.h>
#include "spifuncs.h"
#include <avr/eeprom.h>		/*        */

//////////////////////////
// RAMTRON serial RAM

static uint8_t spinvram_read_byte(void)
{
	fastcount_t i = 8;
	uint8_t v = 0;
	while (i --)
	{
		SCLK_CLR();
		SCLK_SET();

		v <<= 1;
#if defined (__AVR_ATmega128__)
		v |= ((PIND & 0x40) != 0);
#elif defined (__AVR_ATmega32__) || defined (__AVR_ATmega16__)
		v |= ((PIND & 0x01) != 0);
#endif
	}
	return v;
}

static void ramtron_initialize(
	unsigned char target	/* addressing to chip */
	)
{
	
	prog_select(target, 1);	/* start sending data to target chip */
	prog_val(target, 0x06, 8);		/* set write-enable latch */
	prog_select(target, 0);	/* done sending data to target chip */

	prog_select(target, 1);	/* start sending data to target chip */
	prog_val(target, 0x01, 8);		/* set status register data */
	prog_val(target, 0x00, 8);		/* status register data */
	prog_select(target, 0);	/* done sending data to target chip */
}

static unsigned char ramtron_read_status(
	unsigned char target	/* addressing to chip */
	)
{
	unsigned v;

	prog_select(target, 1);	/* start sending data to target chip */
	prog_val(target, 0x05, 8);		/* read status register */
	v = spinvram_read_byte();
	prog_select(target, 0);	/* done sending data to target chip */
	return v;
}

#if defined (NVRAM_TYPE) && ((NVRAM_TYPE == NVRAM_TYPE_FM25L04) || (NVRAM_TYPE == NVRAM_TYPE_AT25L04))
/* 512-byte chips */
static void 
ramtron04_write_nowait(
	unsigned char target,	/* addressing to chip */
	uint16_t addr, 
	void * data, 
	unsigned len)
{
	unsigned i;

	prog_select(target, 1);	/* start sending data to target chip */
	prog_val(target, 0x06, 8);		/* set write-enable latch */
	prog_select(target, 0);	/* done sending data to target chip */

	prog_select(target, 1);	/* start sending data to target chip */
	prog_val(target, (addr > 0xff) ? 0x0a : 0x02, 8);		/* write, a9=0 */
	prog_val(target, addr, 8);
	for (i = 0; i < len; ++ i)
		prog_val(target, ((unsigned char *) data) [i], 8);
	prog_select(target, 0);	/* done sending data to target chip */
}

/* 512-byte chips */
static void 
ramtron04_read_nowait(
	unsigned char target,	/* addressing to chip */
	uint16_t addr, 
	void * data, 
	unsigned len)
{
	unsigned i;

	prog_select(target, 1);	/* start sending data to target chip */
	prog_val(target, (addr > 0xff) ? 0x0b : 0x03, 8);		/* read, a9=0 */
	prog_val(target, addr, 8);
	for (i = 0; i < len; ++ i)
		((unsigned char *) data) [i] = spinvram_read_byte();
	prog_select(target, 0);	/* done sending data to target chip */
}

#endif

#if defined (NVRAM_TYPE) && ((NVRAM_TYPE == NVRAM_TYPE_FM25L16) || (NVRAM_TYPE == NVRAM_TYPE_AT25L16))
/* 2K-byte chips */
static void 
ramtron16_write_nowait(
	unsigned char target,	/* addressing to chip */
	uint16_t addr, 
	void * data, 
	unsigned len)
{
	unsigned i;

	prog_select(target, 1);	/* start sending data to target chip */
	prog_val(target, 0x06, 8);		/* set write-enable latch */
	prog_select(target, 0);	/* done sending data to target chip */

	prog_select(target, 1);	/* start sending data to target chip */
	prog_val(target, 0x02, 8);		/* write */
	prog_val(target, (uint8_t) (addr >> 8), 8);
	prog_val(target, (uint8_t) addr, 8);
	for (i = 0; i < len; ++ i)
		prog_val(target, ((unsigned char *) data) [i], 8);
	prog_select(target, 0);	/* done sending data to target chip */
}

/* 2K-byte chips */
static void 
ramtron16_read_nowait(
	unsigned char target,	/* addressing to chip */
	uint16_t addr, 
	void * data, 
	unsigned len)
{
	unsigned i;

	prog_select(target, 1);	/* start sending data to target chip */
	prog_val(target, 0x03, 8);		/* read */
	prog_val(target, (uint8_t) (addr >> 8), 8);
	prog_val(target, (uint8_t) addr, 8);
	for (i = 0; i < len; ++ i)
		((unsigned char *) data) [i] = spinvram_read_byte();
	prog_select(target, 0);	/* done sending data to target chip */
}

#endif


/*   NVRAM */

void nvram_initialize(void)
{
	ramtron_initialize(targetnvram);
}


void nvram_write(unsigned addr, void * data, unsigned len)
{
#if (NVRAM_TYPE == NVRAM_TYPE_ATMEGA)

		eeprom_busy_wait();
		eeprom_write_block(data, addr, len);

#elif (NVRAM_TYPE == NVRAM_TYPE_AT25L04)

	/*   ~RDY   .  FRAM   .
	    EEPROM */

	while ((ramtron_read_status(targetnvram) & 0x01) != 0)
		continue;

	ramtron04_write_nowait(targetnvram,
		addr, data, len);

#elif (NVRAM_TYPE == NVRAM_TYPE_AT25L16)

	/*   ~RDY   .  FRAM   .
	    EEPROM */

	while ((ramtron_read_status(targetnvram) & 0x01) != 0)
		continue;

	ramtron16_write_nowait(targetnvram,
		addr, data, len);

#elif (NVRAM_TYPE == NVRAM_TYPE_FM25L04)

	ramtron04_write_nowait(targetnvram,
		addr, data, len);

#elif (NVRAM_TYPE == NVRAM_TYPE_FM25L16)

	ramtron16_write_nowait(targetnvram,
		addr, data, len);

#endif
}

void nvram_read(uint16_t addr, void * data, unsigned len)
{
#if (NVRAM_TYPE == NVRAM_TYPE_ATMEGA)

		eeprom_busy_wait();
		eeprom_read_block(data, addr, len);

#elif (NVRAM_TYPE == NVRAM_TYPE_AT25L04)

	/*   ~RDY   .  FRAM   .
	    EEPROM */

	while ((ramtron_read_status(targetnvram) & 0x01) != 0)
		continue;

	ramtron04_read_nowait(targetnvram,
		addr, data, len);

#elif (NVRAM_TYPE == NVRAM_TYPE_AT25L16)

	/*   ~RDY   .  FRAM   .
	    EEPROM */

	while ((ramtron_read_status(targetnvram) & 0x01) != 0)
		continue;

	ramtron16_read_nowait(targetnvram,
		addr, data, len);

#elif (NVRAM_TYPE == NVRAM_TYPE_FM25L04)

	ramtron04_read_nowait(targetnvram,
		addr, data, len);

#elif (NVRAM_TYPE == NVRAM_TYPE_FM25L16)

	ramtron16_read_nowait(targetnvram,
		addr, data, len);

#endif
}

